unit UCVS;

{$mode delphi}

interface

uses
  Classes, SysUtils, UVCS, UBackend;

type

  { TBackendSubversion }

  TBackendCVS = class(TBackend)
    constructor Create;
    function IsWorkingCopy(Directory: string): Boolean; override;
  end;


  { TCVS }

  TCVS = class(TWorkingCopy)
  protected
    procedure Execute(Parameters: array of string);
  public
    procedure Checkout; override;
    procedure Update; override;
    procedure CleanUp; override;
    procedure Commit(Message: TStrings); override;
    procedure Move(Source, Dest: string); override;
    procedure Merge; override;
    procedure Add(FileName: string); override;
    procedure Remove(FileName: string); override;
    procedure GetStatus(FileName: string; Status: TFileStatusList); override;
  end;

  { TRepositoryCVS }

  TRepositoryCVS = class(TRepository)
  protected
    procedure Execute(Parameters: array of string);
  public
    procedure Init; override;
  end;


implementation

{ TRepositoryCVS }

procedure TRepositoryCVS.Execute(Parameters: array of string);
begin
  ExecuteProcess('/usr/bin/cvs', Parameters);
end;

procedure TRepositoryCVS.Init;
begin
  Execute(['-d ' + GetCurrentDir + DirectorySeparator + Path, 'init']);
end;

{ TBackendCVS }

constructor TBackendCVS.Create;
begin
  Name := 'CVS';
  HomePage := 'http://www.nongnu.org/cvs/';
  WorkingCopyClass := TCVS;
  RepositoryClass := TRepositoryCVS;
end;

function TBackendCVS.IsWorkingCopy(Directory: string): Boolean;
begin
  Result := DirectoryExists(Directory + DirectorySeparator + 'CVS');
end;

{ TCVS }

procedure TCVS.Execute(Parameters: array of string);
begin
  EnvVars.Values['CVSROOT'] := GetCurrentDir + DirectorySeparator + Path;
  ExecuteProcess('/usr/bin/cvs', Parameters);
end;

procedure TCVS.Checkout;
begin
  Execute(['checkout', RepositoryURL, Path]);
end;

procedure TCVS.Update;
begin
  Execute(['update']);
end;

procedure TCVS.CleanUp;
begin
  Execute(['cleanup']);
end;

procedure TCVS.Commit(Message: TStrings);
begin
  Execute(['commit', '-m "' + Message.Text + '"']);
end;

procedure TCVS.Move(Source, Dest: string);
begin
  // CVS doesn't support moving files and directories
  RenameFile(Source, Dest);
  Execute(['rm', Source]);
  Execute(['add', Dest]);
end;

procedure TCVS.Merge;
begin
  Execute(['merge']);
end;

procedure TCVS.Add(FileName: string);
begin
  Execute(['add', FileName]);
end;

procedure TCVS.Remove(FileName: string);
begin
  DeleteFile(FileName);
  Execute(['rm', FileName]);
end;

procedure TCVS.GetStatus(FileName: string; Status: TFileStatusList);
begin
  Status.Clear;
  Execute(['status']);

end;

end.

